import unittest
from unittest import mock

from pydactyl import PterodactylClient


class BackupsTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_backups(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/backups',
        }
        self.api.client.servers.backups.list_backups('fds173')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_backup(self, mock_api):
        expected = {
            'endpoint': 'client/servers/abc123/backups',
            'mode': 'POST',
        }
        self.api.client.servers.backups.create_backup('abc123')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_backup_detail(self, mock_api):
        expected = {
            'endpoint': 'client/servers/abc123/backups/longid1111',
        }
        self.api.client.servers.backups.get_backup_detail('abc123',
                                                          'longid1111')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_backup_download(self, mock_api):
        expected = {
            'endpoint': 'client/servers/abc123/backups/longid1111/download',
        }
        self.api.client.servers.backups.get_backup_download('abc123',
                                                            'longid1111')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_backup(self, mock_api):
        expected = {
            'endpoint': 'client/servers/abc123/backups/longid1111',
            'mode': 'DELETE',
        }
        self.api.client.servers.backups.delete_backup('abc123', 'longid1111')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



from pydactyl.api import base


class Databases(base.PterodactylAPI):
    """Pterodactyl Client Server Databases API."""

    def list_databases(self, server_id: str, include_passwords: bool = False):
        """List all databases for a server.

        Optionally includes the database user passwords.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            include_passwords(bool): True to include database user passwords
        """
        params = {}
        if include_passwords:
            params['include'] = 'password'
        endpoint = 'client/servers/{}/databases'.format(server_id)
        response = self._api_request(endpoint=endpoint, params=params)
        return response

    def create_database(self, server_id: str, name: str, remote: str = '%'):
        """Create a new database for the specified server.

        Limits connections to the address specified in remote, defaulting to
        allowing from all.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            remote(str): Remote address to allow connections from (e.g. 1.2.3.4)
        """
        data = {'database': name, 'remote': remote}
        endpoint = 'client/servers/{}/databases'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def rotate_database_password(self, server_id: str, database_id: str):
        """Changes the password of the specified database.

        Generates a new password and returns it in the response.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            database_id(str): Database identifier (long UUID)
        """
        endpoint = 'client/servers/{}/databases/{}/rotate-password'.format(
            server_id, database_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response

    def delete_database(self, server_id: str, database_id: str):
        """Deletes the specified database.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            database_id(str): Database identifier (long UUID)
        """
        endpoint = 'client/servers/{}/databases/{}'.format(server_id,
                                                           database_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response



from pydactyl.api import base


class Files(base.PterodactylAPI):
    """Class for interacting with the Pterodactyl Client API."""

    def list_files(self, server_id: str, path: str = None):
        """List files belonging to specified server.

        Optionally specify a directory and only return results in the
        specified directory.  Directory is relative to the server's root.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            path(str): Path to list in (e.g. 'save_game')
        """
        params = {}
        endpoint = 'client/servers/{}/files/list'.format(server_id)
        if path is not None:
            params = {'directory': path}
        response = self._api_request(endpoint=endpoint, params=params)
        return response

    def download_file(self, server_id: str, path: str) -> str:
        """Get a download link for the specified file on the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            path(str): URL encoded path to desired file (e.g. 'eula.txt')

        Returns:
            response(str): Signed URL to download file
        """
        endpoint = 'client/servers/{}/files/download'.format(server_id)
        params = {'file': path}
        response = self._api_request(endpoint=endpoint, params=params)
        return response.get('attributes').get('url')

    def get_file_contents(self, server_id: str, path: str) -> str:
        """Get contents of the specified file on the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            path(str): URL encoded path to desired file (e.g. 'eula.txt')
        """
        endpoint = 'client/servers/{}/files/contents'.format(server_id)
        params = {'file': path}
        response = self._api_request(endpoint=endpoint, params=params)
        return response

    def rename_file(self, server_id: str, old_name: str, new_name: str,
                    root: str = '/'):
        """Rename a file.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            old_name(str): Name of existing file to rename
            new_name(str): New filename
            root(str): Path to files, relative to server root
        """
        endpoint = 'client/servers/{}/files/rename'.format(server_id)
        data = {'root': root, 'files': [{'from': old_name, 'to': new_name}]}
        response = self._api_request(endpoint=endpoint, mode='PUT', data=data)
        return response

    def copy_file(self, server_id: str, path: str):
        """Makes a copy of a file.

        This is primarily used by the file manager.

        Makes a copy of the file with a unique name.  You cannot specify the
        new name of the file, it just picks one for you.  For example
        'test.txt' will have a copy created named 'test copy.txt'.  Running
        it again will create 'test copy 1.txt'.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            path(str): URL encoded path to desired file (e.g. 'eula.txt')
        """
        endpoint = 'client/servers/{}/files/copy'.format(server_id)
        data = {'location': path}
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def write_file(self, server_id: str, path: str, contents: str):
        """Writes contents to a file.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            path(str): Path to desired file (e.g. 'eula.txt')
            contents(str): Contents to write to the file.
        """
        params = {'file': path}
        endpoint = 'client/servers/{}/files/write'.format(server_id)
        response = self._api_request(
            endpoint=endpoint, mode='POST', params=params, data=contents,
            override_headers={'Content-Type': 'application/text'},
            data_as_json=False)
        return response

    def compress_files(self, server_id: str, files: iter, path: str = '/'):
        """Creates a compressed archive.

        Creates a tar.gz compressed file containing the listed files.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            files(iter): List of files to add to the archive
            path(str): Root path to create the archive from
        """
        data = {'root': path, 'files': files}
        endpoint = 'client/servers/{}/files/compress'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data)
        return response

    def decompress_file(self, server_id: str, file: str, path: str = '/'):
        """Decompresses an archive.

        Decompresses a compressed file to the specified path.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            file(str): Name of the archive file to decompress
            path(str): Root path to decompress in
        """
        data = {'root': path, 'file': file}
        endpoint = 'client/servers/{}/files/decompress'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data)
        return response

    def delete_files(self, server_id: str, files: iter, path: str = '/'):
        """Deletes the specified file(s) or directory(s).

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            files(iter): List of files to delete
            path(str): Root path look for files in
        """
        data = {'root': path, 'files': files}
        endpoint = 'client/servers/{}/files/delete'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data)
        return response

    def create_folder(self, server_id: str, name: str, path: str = '/'):
        """Creates the specified folder in the specified directory.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            name(str): Name of the directory to create
            path(str): Root path to create the directory in
        """
        data = {'root': path, 'name': name}
        endpoint = 'client/servers/{}/files/create-folder'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data)
        return response

    def get_upload_file_url(self, server_id: str) -> str:
        """Returns a signed URL used to upload files to the server using POST.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/files/upload'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response.get('attributes').get('url')



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class SchedulesTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_schedules(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules',
        }
        self.api.client.servers.schedules.list_schedules('srv123')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_schedule(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules',
            'mode': 'POST',
            'data': {'name': 'test', 'minute': '*', 'hour': '1',
                     'day_of_week': 'pants', 'day_of_month': 'doggo',
                     'month': 'may', 'only_when_online': False,
                     'is_active': True},
        }
        self.api.client.servers.schedules.create_schedule(
            'srv123', 'test', '*', '1', 'pants', 'doggo', 'may')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_schedule_details(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/3',
        }
        self.api.client.servers.schedules.get_schedule_details('srv123', 3)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_schedule(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/4',
            'mode': 'POST',
            'data': {'name': 'test', 'minute': '*', 'hour': '1',
                     'day_of_week': 'pants', 'day_of_month': 'doggo',
                     'month': 'may', 'only_when_online': False,
                     'is_active': True},
        }
        self.api.client.servers.schedules.update_schedule(
            'srv123', 4, 'test', '*', '1', 'pants', 'doggo', 'may')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_schedule(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/5',
            'mode': 'DELETE',
        }
        self.api.client.servers.schedules.delete_schedule('srv123', 5)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_task(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/5/tasks',
            'mode': 'POST',
            'data': {'action': 'command', 'payload': 'say Hello World',
                     'time_offset': '6', 'continue_on_failure': False}
        }
        self.api.client.servers.schedules.create_task('srv123', 5, 'command',
                                                      'say Hello World', '6')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_task(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/5/tasks/4',
            'mode': 'POST',
            'data': {'action': 'command', 'payload': 'say Hello World',
                     'time_offset': '6', 'continue_on_failure': True}
        }
        self.api.client.servers.schedules.update_task('srv123', 5, 4, 'command',
                                                      'say Hello World', '6',
                                                      True)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_task(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/5/tasks/4',
            'mode': 'DELETE',
        }
        self.api.client.servers.schedules.delete_task('srv123', 5, 4)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_run_schedule(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/schedules/7/execute',
            'mode': 'POST',
        }
        self.api.client.servers.schedules.run_schedule('srv123', 7)
        mock_api.assert_called_with(**expected)
        

if __name__ == '__main__':
    unittest.main()



from pydactyl.api import base


class Settings(base.PterodactylAPI):
    """Pterodactyl Client Server Settings API."""

    def rename_server(self, server_id: str, name: str):
        """Renames the server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            name(str): New name for the server
        """
        data = {'name': name}
        endpoint = 'client/servers/{}/settings/rename'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def reinstall_server(self, server_id: str):
        """Reinstalls the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/settings/reinstall'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response



from pydactyl.api import base


class Account(base.PterodactylAPI):
    """Class for interacting with the Pterodactyl Client API.

    Methods in this class appear in the base **client.account** namespace
    when using PterodactylClient.
    """

    def get_account(self):
        """List details of the account belonging to the client API key."""
        endpoint = 'client/account'
        response = self._api_request(endpoint=endpoint)
        return response

    def get_2fa_setup_code(self):
        """Generates a TOTP QR code image to allow the setup of 2FA."""
        endpoint = 'client/account/two-factor'
        response = self._api_request(endpoint=endpoint)
        return response

    def enable_2fa(self, code: str):
        """Enables TOTP 2FA.

        Takes a TOTP code generated by a client configured using the code
        from get_2fa_setup_code().  This does not take the code directly and
        you must first setup an authenticator app such as Google
        Authenticator to generate the code provided to this method.

        Args:
            code(str): TOTP code generated by authenticator app
        """
        data = {'code': code}
        endpoint = 'client/account/two-factor'
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def disable_2fa(self, password: str):
        """Disables 2FA on the account.

        Args:
            password(str): User's account password
        """
        data = {'password': password}
        endpoint = 'client/account/two-factor'
        response = self._api_request(endpoint=endpoint, mode='DELETE',
                                     data=data)
        return response

    def update_email(self, email: str, password: str):
        """Updates the email address of the account.

        Args:
            email(str): New email address
            password(str): User's account password
        """
        data = {'email': email, 'password': password}
        endpoint = 'client/account/email'
        response = self._api_request(endpoint=endpoint, mode='PUT',
                                     data=data)
        return response

    def update_password(self, current: str, new: str, new_confirm: str):
        """Updates the password of the account.

        Args:
            current(str): User's current password
            new(str): New password for the account, must match new_confirm
            new_confirm(str): New password for the account, must match new
        """
        data = {'current_password': current, 'password': new,
                'password_confirmation': new_confirm}
        endpoint = 'client/account/password'
        response = self._api_request(endpoint=endpoint, mode='PUT',
                                     data=data)
        return response

    def api_key_list(self):
        """List client's API keys."""
        endpoint = 'client/account/api-keys'
        response = self._api_request(endpoint=endpoint)
        return base.parse_response(response, detail=False)

    def api_key_create(self, description: str, allowed_ips: list):
        """Create a client API key.

        Args:
            description(str): Note for the API key
            allowed_ips(iter): List of allowed IPs
        """
        endpoint = 'client/account/api-keys'
        data = {'description': description, 'allowed_ips': allowed_ips}
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def api_key_delete(self, identifier):
        """Delete a client API key.

        Args:
            identifier(str): API key identifier
        """
        endpoint = 'client/account/api-keys/{}'.format(identifier)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class UsersTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_users(self, mock_api):
        expected = {
            'endpoint': 'client/servers/bofh44/users',
        }
        self.api.client.servers.users.list_users('bofh44')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_user(self, mock_api):
        expected = {
            'endpoint': 'client/servers/bofh44/users',
            'mode': 'POST',
            'data': {'email': 'noob@noob.noob', 'permissions': [
                'control.console']}
        }
        self.api.client.servers.users.create_user('bofh44', 'noob@noob.noob',
                                                  ['control.console'])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_user(self, mock_api):
        expected = {
            'endpoint': 'client/servers/bofh44/users/longuuid',
        }
        self.api.client.servers.users.get_user('bofh44', 'longuuid')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_user(self, mock_api):
        expected = {
            'endpoint': 'client/servers/bofh44/users/longuuid',
            'mode': 'POST',
            'data': {'permissions': ['control.console']}
        }
        self.api.client.servers.users.update_user('bofh44', 'longuuid',
                                                  ['control.console'])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_user(self, mock_api):
        expected = {
            'endpoint': 'client/servers/bofh44/users/longuuid',
            'mode': 'DELETE',
        }
        self.api.client.servers.users.delete_user('bofh44', 'longuuid')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



from pydactyl.api import base


class Users(base.PterodactylAPI):
    """Pterodactyl Client Server Backups API."""

    def list_users(self, server_id: str):
        """List all users added to the server.

        Includes user details and permissions assigned to them.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/users'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def create_user(self, server_id: str, email: str, permissions: iter,
                    username: str = None):
        """Adds a new user to the server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            email(str): Email address of the new user
            permissions(iter): List of permissions to assign to the user
            username(str): Username to assign, randomized if not specified
        """
        data = {'email': email, 'permissions': permissions}
        if username:
            data['username'] = username
        endpoint = 'client/servers/{}/users'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def get_user(self, server_id: str, user_id: str):
        """Retrieves details about the specified user.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            user_id(str): User identifier (long UUID)
        """
        endpoint = 'client/servers/{}/users/{}'.format(server_id, user_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def update_user(self, server_id: str, user_id: str, permissions):
        """Updates the specified user.

        This probably has more options than the documentation list.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            user_id(str): User identifier (long UUID)
            permissions(iter): List of permissions to assign to the user
        """
        data = {'permissions': permissions}
        endpoint = 'client/servers/{}/users/{}'.format(server_id, user_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def delete_user(self, server_id: str, user_id: str):
        """Deletes the specified user.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            user_id(str): User identifier (long UUID)
        """
        endpoint = 'client/servers/{}/users/{}'.format(server_id, user_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response



from pydactyl.api.base import PterodactylAPI
from pydactyl.responses import PaginatedResponse


class Nests(PterodactylAPI):
    """Class for interacting with the Pterdactyl Nests API."""

    def list_nests(self, *include):
        """List all nests.

        Args:
            include(*string): Zero or more include options (eggs, servers)
        """
        params = None if not include else {'include': ','.join(include)}
        endpoint = 'application/nests'
        response = self._api_request(endpoint=endpoint, params=params)
        return PaginatedResponse(self, endpoint, response)

    def get_nest_info(self, nest_id, *include):
        """Get detailed info for the specified nest.

        Args:
            nest_id(int): Pterodactyl Nest ID.
            include(*string): Zero or more include options (eggs, servers)
        """
        params = None if not include else {'include': ','.join(include)}
        response = self._api_request(endpoint='application/nests/%s' % nest_id,
                                     params=params)
        return response

    def get_eggs_in_nest(self, nest_id, *include):
        """Get detailed info for the specified nest.

        Args:
            nest_id(int): Pterodactyl Nest ID.
            include(*string): Zero or more include options (nest, servers, config, script, variables)
        """
        params = None if not include else {'include': ','.join(include)}
        response = self._api_request(
            endpoint='application/nests/%s/eggs' % nest_id,
            params=params)
        return response

    def get_egg_info(self, nest_id, egg_id, *include):
        """Get detailed info for the specified egg.

        Args:
            nest_id(int): Pterodactyl Nest ID.
            egg_id(int): Pterodactyl Egg ID.
            include(*string): Zero or more include options (nest, servers, config, script, variables)
        """
        params = None if not include else {'include': ','.join(include)}
        response = self._api_request(
            endpoint='application/nests/%s/eggs/%s' % (nest_id, egg_id),
            params=params)
        return response



import requests

from pydactyl.api.client.client_api import ClientAPI
from pydactyl.api.locations import Locations
from pydactyl.api.nests import Nests
from pydactyl.api.nodes import Nodes
from pydactyl.api.servers import Servers
from pydactyl.api.user import User
from pydactyl.exceptions import ClientConfigError


def http_adapter(backoff_factor, retries, extra_retry_codes):
    retry_codes = [429] + extra_retry_codes
    retries = requests.packages.urllib3.util.retry.Retry(
        total=retries, status_forcelist=retry_codes,
        backoff_factor=backoff_factor,
        method_whitelist=['DELETE', 'GET', 'HEAD', 'OPTIONS', 'POST', 'PUT'])
    adapter = requests.adapters.HTTPAdapter(max_retries=retries)
    return adapter


class PterodactylClient(object):
    """Provides a simplified interface to the Pterodactyl Panel API.

    Instances of this class allow interaction with the Pterodactyl Panel API.
    """

    def __init__(self, url=None, api_key=None, backoff_factor=1, retries=3,
                 extra_retry_codes=[]):
        """Initialize a Pterodactyl class instance.

        Args:
            url(str): The base URL of the panel to connect to.
            api_key(str): Pterodactyl Panel API key.
            backoff_factor(int): urllib3 retry backoff_factor
            retries(int): maximum number of retries per call
            extra_retry_codes(iter): list of additional integer HTTP status
                    codes to retry on, e.g. [502, 504]
        """
        if not url:
            raise ClientConfigError(
                'You must specify the hostname of a Pterodactyl instance.')

        if not api_key:
            raise ClientConfigError(
                'You must specify a Pterodactyl API key to authenticate.')

        self._api_key = api_key
        self._url = url

        self._session = requests.Session()
        adapter = http_adapter(backoff_factor=backoff_factor,
                               retries=retries,
                               extra_retry_codes=extra_retry_codes)
        self._session.mount('https://', adapter)
        self._session.mount('http://', adapter)

        self._client = None
        self._locations = None
        self._nests = None
        self._nodes = None
        self._servers = None
        self._user = None

    @property
    def client(self):
        self._client = ClientAPI(self._url, self._api_key, self._session)
        return self._client

    @property
    def locations(self):
        self._locations = Locations(self._url, self._api_key, self._session)
        return self._locations

    @property
    def nests(self):
        self._nests = Nests(self._url, self._api_key, self._session)
        return self._nests

    @property
    def nodes(self):
        self._nodes = Nodes(self._url, self._api_key, self._session)
        return self._nodes

    @property
    def servers(self):
        self._servers = Servers(self._url, self._api_key, self._session)
        return self._servers

    @property
    def user(self):
        self._user = User(self._url, self._api_key, self._session)
        return self._user



from pydactyl.api.base import PterodactylAPI
from pydactyl.constants import USE_SSL
from pydactyl.responses import PaginatedResponse


class Nodes(PterodactylAPI):
    """Class for interacting with the Pterdactyl Nodes API."""

    def list_nodes(self, include=None):
        """List all nodes.

        Args:
            include(str): Comma separated list of includes
        """
        endpoint = 'application/nodes'
        params = {}
        if include is not None:
            params['include'] = include
        response = self._api_request(endpoint=endpoint, params=params)
        return PaginatedResponse(self, endpoint, response)

    def get_node_config(self, node_id):
        """Get the Wings configuration for the specified node.

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        response = self._api_request(
            endpoint='application/nodes/{}/configuration'.format(node_id))
        return response

    def get_node_details(self, node_id):
        """Get detailed info for the specified node.

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        response = self._api_request(
            endpoint='application/nodes/{}'.format(node_id))
        return response

    def get_node_info(self, node_id):
        """Get detailed info for the specified node.

        DEPRECATED: Use get_node_details

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        return self.get_node_details(node_id)

    def create_node(self, name, description, location_id, fqdn, memory, disk,
                    memory_overallocate=0,
                    disk_overallocate=0, use_ssl=True, behind_proxy=False,
                    daemon_base='/srv/daemon-data',
                    daemon_sftp=2022, daemon_listen=8080, upload_size=100,
                    public=True, maintenance_mode=False):
        """Creates a new node.

        Args:
            name(str): Node Name, 1-100 characters, valid characters: a-zA-Z0-9_.-[Space]
            description(str): A long description of the node.
            location_id(int): A valid Location ID
            fqdn(str): Domain name used to connect to the daemon.  Alternatively an IP address if not using SSL.
            memory(int): Memory in MB that is available to the daemon for allocation to servers.
            memory_overallocate(int): Percentage of memory that can be overallocated, e.g. 150
            disk(int): Disk space in MB that is available to the daemon for allocation to servers.
            disk_overallocate(int): Percentage of disk space that can be overallocated, e.g. 150
            use_ssl(bool): True to enable SSL, false for insecure HTTP
            behind_proxy(bool): Set to True if running behind a proxy like CloudFlare.  Skips certificate check on boot.
            daemon_base(str): Directory where server files will be stored.
            daemon_sftp(int): Port used by daemon for SFTP.
            daemon_listen(int): Port used by the daemon.
            public(bool): Set to False to prevent servers from being created on this node.
            maintenance_mode(bool): Set to True to disable the node or something.
        """
        data = locals()
        del data['self']
        del data['use_ssl']
        data['scheme'] = USE_SSL[use_ssl]

        response = self._api_request(endpoint='application/nodes',
                                     mode='POST', data=data)
        return response

    def edit_node(self, node_id, name=None, description=None,
                  location_id=None, fqdn=None, use_ssl=None,
                  behind_proxy=None, maintenance_mode=None, memory=None,
                  memory_overallocate=None, disk=None,
                  disk_overallocate=None, upload_size=None, daemon_sftp=None,
                  daemon_listen=None):
        """Update the configuration for an existing node.

        Modifies an existing node identified by node_id and updates any
        parameters that are passed.

        *** WARNING ***
        This endpoint currently requires that you specify all parameters in
        order to edit a node.  This will be updated in the future to
        automatically fetch existing values, however since multiple endpoints
        require this functionality it will be added in a common location.

        Args:
            node_id(int): Pterodactyl Node ID.

            name(str): Node name
            description(str): A long description of the node.  Max 255 characters.
            location_id(int): Location ID
            fqdn(str): Fully qualified domain name of the node
            use_ssl(bool): True to enable SSL, false for insecure HTTP
            behind_proxy(bool): Sets the node's behind_proxy
            maintenance_mode(bool): Set the node's maintenance_mode
            memory(int): Total memory available for the node in MB
            memory_overallocate(int): A percentage to overallocate, e.g. 20 for 120%
            disk(int): Total disk available for the node in MB
            disk_overallocate(int): A percentage to overallocate, e.g. 20 for 120%
            upload_size(int): Maximum size of uploads in file manager in MB
            daemon_sftp(int): Node's SFTP port (default 2022)
            daemon_listen(int): Wings listen port (default 8080)
        """
        data = locals()
        del data['self']
        del data['node_id']
        del data['use_ssl']
        data['scheme'] = USE_SSL[use_ssl]

        response = self._api_request(
            endpoint='application/nodes/{}'.format(node_id), mode='PATCH',
            data=data)
        return response

    def delete_node(self, node_id):
        """Delete an existing node.

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        response = self._api_request(
            endpoint='application/nodes/{}'.format(node_id), mode='DELETE')
        return response

    def list_node_allocations(self, node_id):
        """Retrieves all allocations for a specified node.

        Args:
            node_id(int): Pterodactyl Node ID.

        Returns:
            obj: Iterable response that fetches pages as required.
        """
        endpoint = 'application/nodes/{}/allocations'.format(node_id)
        response = self._api_request(endpoint=endpoint)
        return PaginatedResponse(self, endpoint, response)

    def create_allocations(self, node_id, ip, ports, alias=None):
        """Create one or more allocations.

        Args:
            node_id(int): Pterodactyl Node ID.
            ip(str): The IP address to create the allocation on.
            ports(iter): List of strings representing strings.  Can use
                ranges as supported by Pterodactyl, e.g. ["4000", "4003-4005"]
            alias(str): Optional IP alias.  Used if you want to display a
                different IP address to Panel users, for example to display the
                external IP when behind a NAT.
        """
        data = {'ip': ip, 'ports': ports}
        if alias:
            data['alias'] = alias
        response = self._api_request(
            endpoint='application/nodes/{}/allocations'.format(node_id),
            mode='POST', data=data, json=False)
        return response

    def delete_allocation(self, node_id, allocation_id):
        """Deletes the specified allocation on the specified node.

        Args:
            node_id(int): Pterodactyl Node ID.
            allocation_id(int): Pterodactyl Allocation ID.  This is the
                internal ID assigned to the allocation, not the port number.
        """
        endpoint = 'application/nodes/{}/allocations/{}'.format(node_id,
                                                                allocation_id)
        response = self._api_request(
            endpoint=endpoint, mode='DELETE', json=False)
        return response




from pydactyl.api import base


class Backups(base.PterodactylAPI):
    """Pterodactyl Client Server Backups API."""

    def list_backups(self, server_id: str):
        """List files belonging to specified server.

        Optionally specify a directory and only return results in the
        specified directory.  Directory is relative to the server's root.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/backups'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def create_backup(self, server_id: str):
        """Create a new backup of the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/backups'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response

    def get_backup_detail(self, server_id: str, backup_id: str):
        """Retrieves information about the specified backup.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            backup_id(str): Backup identifier (long UUID)
        """
        endpoint = 'client/servers/{}/backups/{}'.format(server_id, backup_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def get_backup_download(self, server_id: str, backup_id: str):
        """Generates a download link for the specified backup.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            backup_id(str): Backup identifier (long UUID)
        """
        endpoint = 'client/servers/{}/backups/{}/download'.format(server_id,
                                                                  backup_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def delete_backup(self, server_id: str, backup_id: str):
        """Deletes the specified backup.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            backup_id(str): Backup identifier (long UUID)
        """
        endpoint = 'client/servers/{}/backups/{}'.format(server_id, backup_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response



from unittest import main, mock, TestCase

from pydactyl import PterodactylClient


class NodesTests(TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_nodes(self, mock_api):
        expected = {
            'endpoint': 'application/nodes',
            'params': {},
        }
        self.client.nodes.list_nodes()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_nodes_with_includes(self, mock_api):
        expected = {
            'endpoint': 'application/nodes',
            'params': {'include': 'location,other'},
        }
        self.client.nodes.list_nodes(include='location,other')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_node_details(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/11',
        }
        self.client.nodes.get_node_details(11)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_node_config(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/111/configuration',
        }
        self.client.nodes.get_node_config(111)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_node(self, mock_api):
        expected_data = {
            'name': 'Test-Name 1_2.3',
            'description': 'foo bar',
            'location_id': 1,
            'fqdn': 'server1.testy.com',
            'memory': 1024,
            'disk': 5000,
            'memory_overallocate': 0,
            'disk_overallocate': 0,
            'use_ssl': True,
            'behind_proxy': False,
            'daemon_base': '/srv/daemon-data',
            'daemon_sftp': 2022,
            'daemon_listen': 8080,
            'upload_size': 100,
            'public': True,
            'maintenance_mode': False,
        }

        self.client.nodes.create_node(**expected_data)
        del expected_data['use_ssl']
        expected_data['scheme'] = 'https'

        expected = {
            'endpoint': 'application/nodes',
            'mode': 'POST',
            'data': expected_data,
        }

        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_edit_node(self, mock_api):
        expected_data = {
            'name': 'nodey node',
            'description': 'does stuff',
            'location_id': 33,
            'fqdn': 'nodey.nodeynode.com',
            'memory': 1024,
            'disk': 5000,
            'memory_overallocate': 0,
            'disk_overallocate': 0,
            'use_ssl': True,
            'behind_proxy': False,
            'daemon_sftp': 2022,
            'daemon_listen': 8080,
            'upload_size': 222,
            'maintenance_mode': False,
        }

        self.client.nodes.edit_node(11, **expected_data)
        del expected_data['use_ssl']
        expected_data['scheme'] = 'https'

        expected = {
            'endpoint': 'application/nodes/11',
            'mode': 'PATCH',
            'data': expected_data,
        }

        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_node_allocations(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/12/allocations',
        }
        self.client.nodes.list_node_allocations(12)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_allocation_single(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/13/allocations',
            'mode': 'POST',
            'data': {'ip': '10.2.3.4', 'alias': '1.2.3.4', 'ports': ['5000']},
            'json': False,
        }
        self.client.nodes.create_allocations(13, '10.2.3.4', ['5000'],
                                             '1.2.3.4')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_allocation_multiple(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/14/allocations',
            'mode': 'POST',
            'data': {'ip': '10.2.3.4', 'alias': '1.2.3.4', 'ports': ['5000',
                                                                     '5005']},
            'json': False,
        }
        self.client.nodes.create_allocations(14, '10.2.3.4', ['5000', '5005'],
                                             '1.2.3.4')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_allocation_without_alias(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/15/allocations',
            'mode': 'POST',
            'data': {'ip': '1.2.3.4', 'ports': ['5001']},
            'json': False,
        }
        self.client.nodes.create_allocations(
            15, '1.2.3.4', ['5001'])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_allocation(self, mock_api):
        expected = {
            'endpoint': 'application/nodes/16/allocations/123',
            'mode': 'DELETE',
            'json': False,
        }
        self.client.nodes.delete_allocation(16, 123)
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    main()



"""Pydactyl constants."""

__version__ = '1.2.1'
USER_AGENT = 'Pydactyl/' + __version__

POWER_SIGNALS = ('start', 'stop', 'restart', 'kill')
REQUEST_TYPES = ('GET', 'POST', 'PATCH', 'DELETE', 'PUT')
SCHEDULE_ACTIONS = ('command', 'power', 'backup')
USE_SSL = {True: 'https', False: 'http'}



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class FilesTest(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_files(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/list',
            'params': {},
        }
        self.api.client.servers.files.list_files('gudsrvr')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_files_with_path(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/list',
            'params': {'directory': 'saves/backups/'},
        }
        self.api.client.servers.files.list_files('gudsrvr', 'saves/backups/')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_download_file(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/download',
            'params': {'file': 'backups/today.zip'},
        }
        self.api.client.servers.files.download_file('gudsrvr',
                                                    'backups/today.zip')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_file_contents(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/contents',
            'params': {'file': 'backups/today.zip'},
        }
        self.api.client.servers.files.get_file_contents('gudsrvr',
                                                        'backups/today.zip')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_rename_file(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/rename',
            'mode': 'PUT',
            'data': {'root': '/', 'files': [{'from': 'old.txt',
                                             'to': 'new.txt'}]},
        }
        self.api.client.servers.files.rename_file('gudsrvr', 'old.txt',
                                                  'new.txt')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_rename_file_with_root(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/rename',
            'mode': 'PUT',
            'data': {'root': 'backups', 'files': [{'from': 'today.zip',
                                                   'to': 'yesterday.zip'}]},
        }
        self.api.client.servers.files.rename_file('gudsrvr', 'today.zip',
                                                  'yesterday.zip', 'backups')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_copy_file(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/copy',
            'mode': 'POST',
            'data': {'location': 'config.json'},
        }
        self.api.client.servers.files.copy_file('gudsrvr', 'config.json')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_write_file(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/write',
            'mode': 'POST',
            'params': {'file': 'eula.txt'},
            'data': 'All your base are belong to us',
            'data_as_json': False,
            'override_headers': {'Content-Type': 'application/text'},
        }
        self.api.client.servers.files.write_file(
            'gudsrvr', 'eula.txt', 'All your base are belong to us')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_compress_files(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/compress',
            'mode': 'POST',
            'data': {'root': '/', 'files': ['game.map', 'game.save']},
        }
        self.api.client.servers.files.compress_files(
            'gudsrvr', ['game.map', 'game.save'])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_compress_files_with_path(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/compress',
            'mode': 'POST',
            'data': {'root': 'backups', 'files': ['today.zip',
                                                  'yesterday.zip']},
        }
        self.api.client.servers.files.compress_files(
            'gudsrvr', ['today.zip', 'yesterday.zip'], 'backups')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_decompress_files(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/decompress',
            'mode': 'POST',
            'data': {'root': '/', 'file': 'backup.tar.gz'},
        }
        self.api.client.servers.files.decompress_file(
            'gudsrvr', 'backup.tar.gz')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_decompress_files_with_path(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/decompress',
            'mode': 'POST',
            'data': {'root': 'gamefiles', 'file': 'backup.tar.gz'},
        }
        self.api.client.servers.files.decompress_file(
            'gudsrvr', 'backup.tar.gz', 'gamefiles')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_files(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/delete',
            'mode': 'POST',
            'data': {'root': '/', 'files': ['oldbackup.zip', 'oldconfig.txt']},
        }
        self.api.client.servers.files.delete_files(
            'gudsrvr', ['oldbackup.zip', 'oldconfig.txt'])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_files_with_path(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/delete',
            'mode': 'POST',
            'data': {'root': 'savegames', 'files': ['save1', 'save2']},
        }
        self.api.client.servers.files.delete_files(
            'gudsrvr', ['save1', 'save2'], 'savegames')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_folder(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/create-folder',
            'mode': 'POST',
            'data': {'root': '/', 'name': 'folders_and_dirs_are_not_the_same'},
        }
        self.api.client.servers.files.create_folder(
            'gudsrvr', 'folders_and_dirs_are_not_the_same')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_folder_with_path(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/create-folder',
            'mode': 'POST',
            'data': {'root': 'backups', 'name': 'archived'},
        }
        self.api.client.servers.files.create_folder(
            'gudsrvr', 'archived', 'backups')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_upload_file_url(self, mock_api):
        expected = {
            'endpoint': 'client/servers/gudsrvr/files/upload',
        }
        self.api.client.servers.files.get_upload_file_url('gudsrvr')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class StartupTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_variables(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/startup',
        }
        self.api.client.servers.startup.list_variables('srv123')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_variable(self, mock_api):
        expected = {
            'endpoint': 'client/servers/srv123/startup/variable',
            'mode': 'PUT',
            'data': {'key': 'varname', 'value': 'someval'},
        }
        self.api.client.servers.startup.update_variable('srv123', 'varname',
                                                        'someval')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class AccountTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_account(self, mock_api):
        expected = {
            'endpoint': 'client/account',
        }
        self.api.client.account.get_account()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_2fa_setup_code(self, mock_api):
        expected = {
            'endpoint': 'client/account/two-factor',
        }
        self.api.client.account.get_2fa_setup_code()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_enable_2fa(self, mock_api):
        expected = {
            'endpoint': 'client/account/two-factor',
            'mode': 'POST',
            'data': {'code': '123456'},
        }
        self.api.client.account.enable_2fa('123456')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_disable_2fa(self, mock_api):
        expected = {
            'endpoint': 'client/account/two-factor',
            'mode': 'DELETE',
            'data': {'password': '123456'},
        }
        self.api.client.account.disable_2fa('123456')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_email(self, mock_api):
        expected = {
            'endpoint': 'client/account/email',
            'mode': 'PUT',
            'data': {'email': 'me@me.com', 'password': 'hunter2'},
        }
        self.api.client.account.update_email('me@me.com', 'hunter2')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_update_password(self, mock_api):
        expected = {
            'endpoint': 'client/account/password',
            'mode': 'PUT',
            'data': {'current_password': 'hunter2', 'password': 'hunter3',
                     'password_confirmation': 'hunter3'},
        }
        self.api.client.account.update_password('hunter2', 'hunter3', 'hunter3')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_api_keys(self, mock_api):
        expected = {
            'endpoint': 'client/account/api-keys',
        }
        self.api.client.account.api_key_list()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_api_key(self, mock_api):
        expected = {
            'endpoint': 'client/account/api-keys',
            'mode': 'POST',
            'data': {'description': 'Test key', 'allowed_ips': ["127.0.0.1"]},
        }
        self.api.client.account.api_key_create('Test key', ["127.0.0.1"])
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_api_key(self, mock_api):
        expected = {
            'endpoint': 'client/account/api-keys/abc123',
            'mode': 'DELETE',
        }
        self.api.client.account.api_key_delete('abc123')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class NetworkTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_allocations(self, mock_api):
        expected = {
            'endpoint': 'client/servers/notwork/network/allocations',
        }
        self.api.client.servers.network.list_allocations('notwork')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_assign_allocation(self, mock_api):
        expected = {
            'endpoint': 'client/servers/notwork/network/allocations',
            'mode': 'POST',
        }
        self.api.client.servers.network.assign_allocation('notwork')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_set_allocation_note(self, mock_api):
        expected = {
            'endpoint': 'client/servers/notwork/network/allocations/2',
            'mode': 'POST',
            'data': {'notes': 'some note'},
        }
        self.api.client.servers.network.set_allocation_note('notwork', 2,
                                                            'some note')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_set_primary_allocation(self, mock_api):
        expected = {
            'endpoint': 'client/servers/notwork/network/allocations/33/primary',
            'mode': 'POST',
        }
        self.api.client.servers.network.set_primary_allocation('notwork', 33)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_unassign_allocation(self, mock_api):
        expected = {
            'endpoint': 'client/servers/notwork/network/allocations/44',
            'mode': 'DELETE',
        }
        self.api.client.servers.network.unassign_allocation('notwork', 44)
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



import requests

from pydactyl.constants import REQUEST_TYPES
from pydactyl.exceptions import BadRequestError
from pydactyl.exceptions import PterodactylApiError


def parse_response(response, detail=False):
    """Parse the response data.

    Optionally includes additional data that specifies the object type
    and requires accessing the data through a nested dictionary.  The
    Client API doesn't include any additional information, but the
    Servers API includes created and updated timestamps in the detailed
    response.

    Args:
         response(dict): A request response object.
         detail(bool): Include additional data from the raw response.
    """
    if detail:
        data = response
    else:
        if response['object'] == 'list':
            data = [item.get('attributes') for item in response.get('data')]
        else:
            data = response.get('attributes')

    return data


def url_join(*args):
    """Join combine URL parts to get the full endpoint address."""
    return '/'.join(arg.strip('/') for arg in args)


class PterodactylAPI(object):
    """Pterodactyl API client."""

    def __init__(self, url, api_key, session=None):
        self._api_key = api_key
        self._url = url
        self._session = session or requests.Session()

    def _get_headers(self):
        """Headers to use for API calls."""
        headers = {
            'Authorization': 'Bearer {0}'.format(self._api_key),
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }
        return headers

    def _api_request(self, endpoint, mode='GET', params=None, data=None,
                     json=None, override_headers=None, data_as_json=True):
        """Make a request to the Pterodactyl API.

        Args:
            endpoint(str): URI for the API
            mode(str): Request type, one of ('GET', 'POST', 'PATCH',
                    'DELETE', 'PUT')
            params(dict): Extra parameters to pass to the endpoint,
                    e.g. a query string
            data(dict): POST data
            json(bool): Set to False to return the response object,
                    True for just JSON.  Defaults to returning JSON if possible
                    otherwise the response object.
            override_headers(dict): Headers to override, e.g. to set the
                    Content-Type
            data_as_json(bool): If True data will be posted as JSON

        Returns:
            response: A HTTP response object or the JSON response depending on
                    the value of the json parameter.
        """
        if not endpoint:
            raise BadRequestError('No API endpoint was specified.')

        url = url_join(self._url, 'api', endpoint)
        headers = self._get_headers()
        if override_headers:
            headers.update(override_headers)

        if mode == 'GET':
            response = self._session.get(url, params=params, headers=headers)
        elif mode == 'POST':
            if data_as_json:
                response = self._session.post(url, params=params,
                                              headers=headers, json=data)
            else:
                response = self._session.post(url, params=params,
                                              headers=headers, data=data)
        elif mode == 'PATCH':
            response = self._session.patch(url, params=params, headers=headers,
                                           json=data)
        elif mode == 'DELETE':
            response = self._session.delete(url, params=params, headers=headers)
        elif mode == 'PUT':
            response = self._session.put(url, params=params, headers=headers,
                                         json=data)
        else:
            raise BadRequestError(
                'Invalid request type specified(%s).  Must be one of %r.' % (
                    mode, REQUEST_TYPES))

        try:
            response_json = response.json()
        except ValueError:
            response_json = {}

        if response.status_code in (400, 422):
            raise PterodactylApiError('API Request resulted in errors: %s' %
                                      response_json.get('errors'))
        else:
            response.raise_for_status()

        if json is True:
            return response_json
        elif json is False:
            return response
        else:
            return response_json or response



from pydactyl.api import base


class Startup(base.PterodactylAPI):
    """Pterodactyl Client Server Startup API."""

    def list_variables(self, server_id: str):
        """Lists all variables on the server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/startup'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def update_variable(self, server_id: str, name: str, value: str):
        """Updates the specified server variable.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            name(str): Variable name to update
            value(str): Value to assign to the updated variable
        """
        data = {'key': name, 'value': value}
        endpoint = 'client/servers/{}/startup/variable'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='PUT', data=data)
        return response



"""Classes used for creating responses."""


class PaginatedResponse(object):
    """An iterable API response that returns paginated results."""

    def __init__(self, client, endpoint, data):
        self._client = client
        self.data = data['data']
        self.endpoint = endpoint
        self.meta = data['meta']

    def __getitem__(self, item):
        if isinstance(item, int):
            return self.data[item]
        if hasattr(self, item):
            return getattr(self, item, None)
        raise KeyError(item)

    def __iter__(self):
        if hasattr(self, '_iteration'):
            return iter(self.data)
        else:
            self._iteration = 0
            return self

    def __next__(self):
        """"Retrieves the next page of results.

        Returns:
            object: An instance of PaginatedResponse

        Raises:
            PterodactylApiError: If the API request fails
            StopIteration: When the next page does not exist.
        """
        self._iteration += 1
        if self._iteration == 1:
            # PaginatedResponses are initialized with the first page of results
            return self
        if self._next_page_exists(self.meta):
            params = {'page': self._iteration}
            response = self._client._api_request(endpoint=self.endpoint,
                                                 params=params)
            self.data = response['data']
            self.meta = response['meta']
            return self
        else:
            raise StopIteration

    def __str__(self):
        return '{}'.format(self.data)

    def get(self, key, default=None):
        """Retrieves a key from the paginated response.

        Returns:
            Value for the specified key
        """
        return getattr(self, key, default)

    def collect(self):
        """Collect all results from all pages.

        Returns:
            iter: Combined responses from all pages
        """
        collected = []
        for page in self:
            collected.extend(page.data)
        return collected

    def get_next_page_link(self):
        """Get a link to the next page.

        Returns:
            str: Link provided by API
        """
        return self.meta['pagination']['links'].get('next')

    def get_previous_page_link(self):
        """Get a link to the previous page.

        Returns:
            str: Link provided by API
        """
        return self.meta['pagination']['links'].get('previous')

    @staticmethod
    def _next_page_exists(data):
        """Determine if the response contains a valid link to the next page.

        Returns:
            bool: True if next page exists.
        """
        exists = ('pagination' in data
                  and 'links' in data['pagination']
                  and 'next' in data['pagination']['links']
                  and data['pagination']['links']['next'] != '')
        return exists



# Client Account APIs
from pydactyl.api.client.account.base import Account

# Client Server APIs
from pydactyl.api.client.servers.backups import Backups
from pydactyl.api.client.servers.base import ServersBase
from pydactyl.api.client.servers.databases import Databases
from pydactyl.api.client.servers.files import Files
from pydactyl.api.client.servers.network import Network
from pydactyl.api.client.servers.schedules import Schedules
from pydactyl.api.client.servers.settings import Settings
from pydactyl.api.client.servers.startup import Startup
from pydactyl.api.client.servers.users import Users

from pydactyl.api.base import PterodactylAPI


class ClientAPI(PterodactylAPI):
    """Provides a simplified interface to the Pterodactyl Panel API.

    This class is only used by PterodactylClient.  It provides an interface
    for the Client API endpoints.
    """

    def __init__(self, *args, **kwargs):
        """Initialize a Pterodactyl class instance."""
        self._account = None
        self._servers = None
        super().__init__(*args, **kwargs)

    @property
    def account(self):
        self._account = Account(self._url, self._api_key)
        return self._account

    @property
    def servers(self):
        self._servers = ClientServersAPI(self._url, self._api_key)
        return self._servers


class ClientServersAPI(ServersBase, ClientAPI):
    """Provides a simplified interface to the Pterodactyl Panel API.

    This class is only used by PterodactylClient.  It provides an interface
    for the Client API endpoints.
    """

    def __init__(self, *args, **kwargs):
        """Initialize a Pterodactyl class instance."""
        self._backups = None
        self._databases = None
        self._files = None
        self._network = None
        self._schedules = None
        self._settings = None
        self._startup = None
        self._users = None
        super().__init__(*args, **kwargs)

    @property
    def backups(self):
        self._backups = Backups(self._url, self._api_key)
        return self._backups

    @property
    def databases(self):
        self._databases = Databases(self._url, self._api_key)
        return self._databases

    @property
    def files(self):
        self._files = Files(self._url, self._api_key)
        return self._files

    @property
    def network(self):
        self._network = Network(self._url, self._api_key)
        return self._network

    @property
    def schedules(self):
        self._schedules = Schedules(self._url, self._api_key)
        return self._schedules

    @property
    def settings(self):
        self._settings = Settings(self._url, self._api_key)
        return self._settings

    @property
    def startup(self):
        self._startup = Startup(self._url, self._api_key)
        return self._startup

    @property
    def users(self):
        self._users = Users(self._url, self._api_key)
        return self._users



from pydactyl.api import base
from pydactyl.constants import SCHEDULE_ACTIONS
from pydactyl.exceptions import BadRequestError


def check_schedule_action_valid(action):
    if action not in SCHEDULE_ACTIONS:
        raise BadRequestError(
            'Invalid schedule action sent({}), must be one of: {}'.format(
                action, SCHEDULE_ACTIONS))


class Schedules(base.PterodactylAPI):
    """Pterodactyl Client Server Databases API."""

    def list_schedules(self, server_id: str):
        """List all schedules for a server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/schedules'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def create_schedule(self, server_id: str, name: str, minute: str,
                        hour: str, day_of_week: str, day_of_month: str,
                        month: str, is_active: bool = True, 
                        only_when_online: bool = False):
        """Creates a new schedule for the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            name(str): Friendly name for the schedule
            minute(str): Value for Cron minute field
            hour(str): Value for Cron hour field
            day_of_week(str): Value for Cron day_of_week field
            day_of_month(str): Value for Cron day_of_month field
            month(str): Value for Cron month field
            is_active(bool): False to create the schedule as disabled
            only_when_online(bool): True to run schedule only when server is running
        """
        data = {'name': name, 'minute': minute, 'hour': hour,
                'day_of_week': day_of_week, 'day_of_month': day_of_month,
                'month': month, 'is_active': is_active, 
                'only_when_online': only_when_online}
        endpoint = 'client/servers/{}/schedules'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def get_schedule_details(self, server_id: str, schedule_id: int):
        """Retrieves the specified schedule.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)
        """
        endpoint = 'client/servers/{}/schedules/{}'.format(server_id,
                                                           schedule_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def update_schedule(self, server_id: str, schedule_id: int, name: str,
                        minute: str, hour: str, day_of_week: str,
                        day_of_month: str, month: str, is_active: bool = True, 
                        only_when_online: bool = False):
        """Updates the specified schedule.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)

            name(str): Friendly name for the schedule
            minute(str): Value for Cron minute field
            hour(str): Value for Cron hour field
            day_of_week(str): Value for Cron day_of_week field
            day_of_month(str): Value for Cron day_of_month field
            month(str): Value for Cron month field
            is_active(bool): False to create the schedule as disabled
            only_when_online(bool): True to run schedule only when server is running
        """
        data = {'name': name, 'minute': minute, 'hour': hour,
                'day_of_week': day_of_week, 'day_of_month': day_of_month,
                'month': month, 'is_active': is_active, 
                'only_when_online': only_when_online}
        endpoint = 'client/servers/{}/schedules/{}'.format(server_id,
                                                           schedule_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def delete_schedule(self, server_id: str, schedule_id: int):
        """Deletes the specified schedule.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)
        """
        endpoint = 'client/servers/{}/schedules/{}'.format(server_id,
                                                           schedule_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response

    def create_task(self, server_id: str, schedule_id: int, action: str,
                    payload: str, time_offset: str = '0', 
                    continue_on_failure: bool = False):
        """Creates a new task on the specified schedule.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)
            action(str): Type of action to use
            payload(str): Payload to send for the action
            time_offset(str): Offset in seconds
            continue_on_failure(bool): True to continue schedule run if this task fails
        """
        check_schedule_action_valid(action)
        data = {'action': action, 'payload': payload,
                'time_offset': time_offset, 'continue_on_failure': continue_on_failure}
        endpoint = 'client/servers/{}/schedules/{}/tasks'.format(server_id,
                                                                 schedule_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def update_task(self, server_id: str, schedule_id: int, task_id: int,
                    action: str, payload: str, time_offset: str = '0', 
                    continue_on_failure: bool = False):
        """Updates the specified task.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)
            task_id(int): Task identifier (e.g. 4)
            action(str): Type of action to use
            payload(str): Payload to send for the action
            time_offset(str): Offset in seconds
            continue_on_failure(bool): True to continue schedule run if this task fails
        """
        check_schedule_action_valid(action)
        data = {'action': action, 'payload': payload,
                'time_offset': time_offset, 'continue_on_failure': continue_on_failure}
        endpoint = 'client/servers/{}/schedules/{}/tasks/{}'.format(
            server_id, schedule_id, task_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def delete_task(self, server_id: str, schedule_id: int, task_id: int):
        """Deletes the specified task.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier (e.g. 2)
            task_id(int): Task identifier (e.g. 4)
        """
        endpoint = 'client/servers/{}/schedules/{}/tasks/{}'.format(
            server_id, schedule_id, task_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response

    def run_schedule(self, server_id: str, schedule_id: int):
        """Runs the specified schedule.
        
        Args:
            server_id(str): Server identifier (abbreviated UUID)
            schedule_id(int): Schedule identifier
        """
        endpoint = 'client/servers/{}/schedules/{}/execute'.format(
            server_id, schedule_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response



from unittest import main, mock, TestCase

from pydactyl import PterodactylClient


class NestsTests(TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_nests(self, mock_api):
        expected = {
            'endpoint': 'application/nests',
        }
        self.client.nests.list_nests()
        mock_api.assert_called_with(**expected, params=None)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_nests_with_includes(self, mock_api):
        expected = {
            'endpoint': 'application/nests',
        }
        self.client.nests.list_nests('eggs', 'servers')
        mock_api.assert_called_with(
            **expected, params={'include': 'eggs,servers'})

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def get_nest_info(self, mock_api):
        expected = {
            'endpoint': 'application/nests/11',
        }
        self.client.nests.get_nest_info(11)
        mock_api.assert_called_with(**expected, params=None)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def get_nest_info_with_include(self, mock_api):
        expected = {
            'endpoint': 'application/nests/11',
        }
        self.client.nests.get_nest_info(11, 'config')
        mock_api.assert_called_with(**expected, params={'include': 'config'})

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_eggs_in_nest(self, mock_api):
        expected = {
            'endpoint': 'application/nests/22/eggs',
        }
        self.client.nests.get_eggs_in_nest(22)
        mock_api.assert_called_with(**expected, params=None)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_eggs_in_nest_with_includes(self, mock_api):
        expected = {
            'endpoint': 'application/nests/22/eggs',
        }
        self.client.nests.get_eggs_in_nest(22, 'nest', 'config')
        mock_api.assert_called_with(
            **expected, params={'include': 'nest,config'})

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_egg_info(self, mock_api):
        expected = {
            'endpoint': 'application/nests/33/eggs/44',
        }
        self.client.nests.get_egg_info(33, 44)
        mock_api.assert_called_with(**expected, params=None)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_egg_info_with_includes(self, mock_api):
        expected = {
            'endpoint': 'application/nests/33/eggs/44',
        }
        self.client.nests.get_egg_info(33, 44, 'servers', 'config')
        mock_api.assert_called_with(
            **expected, params={'include': 'servers,config'})


if __name__ == '__main__':
    main()



import unittest
from copy import deepcopy
from unittest import mock

from pydactyl import PterodactylClient
from pydactyl.responses import PaginatedResponse

TEST_META = {
    'pagination':
        {'total': 106, 'count': 25, 'per_page': 25, 'current_page': 1,
         'total_pages': 5, 'links': {
            'next': 'https://panel.mydomain.com/api/application/nodes/1'
                    '/allocations?page=3',
            'previous': 'https://panel.mydomain.com/api/application/nodes/1'
                        '/allocations?page=1'}}}
TEST_DATA = {'data': [{'dummy': 'data1'}, {'dummy': 'data2'}], 'meta':
    TEST_META}

MULTIPAGE_TEST_DATA = [
    {'data': ['thing1', 'thing2', 'sometimespage1isweird'], 'meta': TEST_META},
    {'data': ['thing3', 'thing4'], 'meta': TEST_META},
    {'data': ['thing84', 'thing97', 'fdas', 'pepperoni'], 'meta': TEST_META},
    {'data': ['one', 'two', 'three', 4, 5, '6'], 'meta': TEST_META},
    {'data': ['pineappleisanacceptablepizzatopping'], 'meta': TEST_META},
]


class PaginatedResponseTests(unittest.TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    def test_paginated_response_init(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)

        self.assertEqual(response[0]['dummy'], 'data1')
        self.assertEqual(response[1]['dummy'], 'data2')

    def test_paginated_response_get(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(response.get('meta'), TEST_META)

    def test_paginated_response_get_with_default(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(response.get('badname', 'test_default'),
                         'test_default')

    def test_paginated_response_get_next_page_link(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(
            response.get_next_page_link(),
            'https://panel.mydomain.com/api/application/nodes/1/allocations'
            '?page=3')

    def test_paginated_response_get_previous_page_link(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(
            response.get_previous_page_link(),
            'https://panel.mydomain.com/api/application/nodes/1/allocations'
            '?page=1')

    def test_paginated_response_next_page_exists(self):
        self.assertTrue(PaginatedResponse._next_page_exists(TEST_META))

    def test_paginated_response_next_page_does_not_exist(self):
        data = deepcopy(TEST_META)
        self.assertTrue(PaginatedResponse._next_page_exists(data))
        del data['pagination']['links']['next']
        self.assertFalse(PaginatedResponse._next_page_exists(data))

    def test_paginated_response_fetches_first_page(self):
        expected = {
            'endpoint': 'api/asdf',
            'params': {'page': 2},
        }
        self.client._api_request = mock.MagicMock()
        response = PaginatedResponse(self.client, 'api/asdf', TEST_DATA)
        for _ in response:
            continue

        self.client._api_request.assert_called_with(**expected)

    def test_paginated_response_data_property_returns_data(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(response.data, TEST_DATA['data'])

    def test_paginated_response_str_method(self):
        response = PaginatedResponse(self.client, 'anyendpoint', TEST_DATA)
        self.assertEqual(str(response), str(TEST_DATA['data']))

    def test_paginated_response_collect_method(self):
        self.client._api_request = mock.MagicMock()
        self.client._api_request.side_effect = MULTIPAGE_TEST_DATA[1:]
        response = PaginatedResponse(self.client, 'asdf',
                                     MULTIPAGE_TEST_DATA[0])
        self.assertListEqual(
            response.collect(),
            [item for data in MULTIPAGE_TEST_DATA for item in data['data']])

    def test_paginated_response_multipage_iterator(self):
        self.client._api_request = mock.MagicMock()
        self.client._api_request.side_effect = MULTIPAGE_TEST_DATA[1:]
        response = PaginatedResponse(self.client, 'asdf',
                                     MULTIPAGE_TEST_DATA[0])
        expected = [item for data in MULTIPAGE_TEST_DATA for item in
                    data['data']]
        self.assertListEqual(expected,
                             [item for page in response for item in page])

    def test_paginated_response_has_correct_number_of_items(self):
        self.client._api_request = mock.MagicMock()
        self.client._api_request.side_effect = MULTIPAGE_TEST_DATA[1:]
        response = PaginatedResponse(self.client, 'asdf',
                                     MULTIPAGE_TEST_DATA[0])
        expected = [item for data in MULTIPAGE_TEST_DATA for item in
                    data['data']]
        expected_count = 16
        self.assertEqual(expected_count, len(expected))
        self.assertEqual(expected_count, len(response.collect()))



from pydactyl.api import base
from pydactyl.exceptions import BadRequestError
from pydactyl.responses import PaginatedResponse


class Servers(base.PterodactylAPI):
    """Class for interacting with the Pterdactyl Servers API."""

    def list_servers(self):
        """List all servers."""
        endpoint = 'application/servers'
        response = base.parse_response(self._api_request(endpoint=endpoint),
                                       detail=True)
        return PaginatedResponse(self, endpoint, response)

    def get_server_info(self, server_id=None, external_id=None, detail=False):
        """Get detailed info for the specified server.

        Args:
            server_id(int): Pterodactyl Server ID.
            external_id(int): Server ID from an external system like WHMCS
            detail(bool): If True includes created and updated timestamps.
        """
        if not server_id and not external_id:
            raise BadRequestError('Must specify either server_id or '
                                  'external_id.')
        if server_id and external_id:
            raise BadRequestError('Specify either server_id or external_id, '
                                  'not both.')

        if server_id:
            endpoint = 'application/servers/%s' % server_id
        else:
            endpoint = 'application/servers/external/%s' % external_id

        response = self._api_request(endpoint=endpoint)
        return base.parse_response(response, detail)

    def suspend_server(self, server_id):
        """Suspend the server with the specified internal ID.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        response = self._api_request(
            endpoint='application/servers/%s/suspend' % server_id, mode='POST')
        return response

    def unsuspend_server(self, server_id):
        """Suspend the server with the specified internal ID.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        response = self._api_request(
            endpoint='application/servers/%s/unsuspend' % server_id,
            mode='POST')
        return response

    def reinstall_server(self, server_id):
        """Reinstall the server with the specified internal ID.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        response = self._api_request(
            endpoint='application/servers/%s/reinstall' % server_id,
            mode='POST')
        return response

    def rebuild_server(self, server_id):
        """Rebuild the server with the specified internal ID.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        response = self._api_request(
            endpoint='application/servers/%s/rebuild' % server_id,
            mode='POST')
        return response

    def delete_server(self, server_id, force=False):
        """Delete the server with the specified internal ID.

        Attempts to delete the server from both the panel and daemon.
        By default if either one reports an error the action will be cancelled.

        Args:
            server_id(int): Pterodactyl Server ID.
            force(bool): If True the delete action will continue if the panel or
                    daemon reports an error.
        """
        endpoint = 'application/servers/%s' % server_id
        if force:
            endpoint += '/force'

        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response

    def list_server_databases(self, server_id):
        """List the database servers assigned to the specified server ID.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        endpoint = 'application/servers/%s/databases' % server_id
        response = self._api_request(endpoint=endpoint)
        return PaginatedResponse(self, endpoint, response)

    def get_server_database_info(self, server_id, database_id, detail=False):
        """Get information about the specified database on the specified server.

        Args:
            server_id(int): Pterodactyl Server ID.
            database_id(int): Database ID for specified server.
            detail(bool): If True includes the object type and a nested data
                    structure.
        """
        response = self._api_request(
            endpoint='application/servers/%s/databases/%s' % (server_id,
                                                              database_id))
        return base.parse_response(response, detail)

    def create_server_database(self, server_id):
        """Create a database for the specified server.

        Args:
            server_id(int): Pterodactyl Server ID.
        """
        response = self._api_request(
            endpoint='application/servers/%s/databases' % server_id,
            mode='POST')
        return response

    def delete_server_database(self, server_id, database_id):
        """Delete the specified database for the specified server.

        Args:
            server_id(int): Pterodactyl Server ID.
            database_id(int): Database ID for specified server.
        """
        response = self._api_request(
            endpoint='application/servers/%s/databases/%s' % (server_id,
                                                              database_id),
            mode='DELETE')
        return response

    def reset_server_database_password(self, server_id, database_id):
        """Resets the password for the specified server database.

        Args:
            server_id(int): Pterodactyl Server ID.
            database_id(int): Database ID for specified server.
        """
        response = self._api_request(
            endpoint='application/servers/%s/databases/%s/reset-password' % (
                server_id,
                database_id),
            mode='POST')
        return response

    def create_server(self, name, user_id, nest_id, egg_id, memory_limit,
                      swap_limit, disk_limit, location_ids=[], port_range=[],
                      environment={}, cpu_limit=0, io_limit=500,
                      database_limit=0, allocation_limit=0, backup_limit=0,
                      docker_image=None, startup_cmd=None, dedicated_ip=False,
                      start_on_completion=True, oom_disabled=True,
                      default_allocation=None, additional_allocations=None):
        """Creates one or more servers in the specified locations.

        Creates server instance(s) and begins the install process using the
        specified configurations and limits.  If more than one value is
        specified for location_ids then identical servers will be created in
        each location.

        Args:
            name(str): Name of the server to display in the panel.
            user_id(int): User ID that will own the server.
            nest_id(int): Nest ID for the created server.
            egg_id(int): Egg ID for the created server.
            memory_limit(int): Memory limit in MB for the Docker container.  To
                    allow unlimited memory limit set to 0.
            swap_limit(int): Swap limit in MB for the Docker container.  To not
                    assign any swap set to 0.  For unlimited swap set to -1.
            disk_limit(int): Disk limit in MB for the Docker container.  To
                    allow unlimited disk space set to 0.
            environment(dict): Key value pairs of Service Variables to set.
                    Every variable from the egg must be set or the API will
                    return an error.  Default values will be pulled from the egg
                    config or set to None.
            location_ids(iter): List of location_ids where the server(s) will be
                    created.  If more than one location is specified
                    identical servers will be created at each.
            port_range(iter): List of ports or port ranges to use when
                    selecting an allocation.  If empty, all ports will be
                    considered.  If set, only ports appearing in the list or
                    range will be used.  e.g. [20715, '20815-20915']
            cpu_limit(int): CPU limit for the Docker container.  To allow
                    unlimited CPU usage set to 0.  To limit to one core set
                    to 100.  For four cores set to 400.
            io_limit(int): Block IO weight for the Docker container.
                    Must be between 10 and 1000.
            database_limit(int): Maximum number of databases that can be
                    assigned to this server.
            allocation_limit(int): Maximum number of allocations that can be
                    assigned to this server.
            backup_limit(int): Maximum number of backups that can be
                    created for this server.
            docker_image(str): Name or URL of the Docker server to use.
                    e.g. quay.io/pterodactyl/core:java-glibc
            startup_cmd(str): Startup command, if specified replaces the
                    egg's default value.
            dedicated_ip(bool): Limit allocations to IPs without any existing
                    allocations.
            start_on_completion(bool): Start server after install completes.
            oom_disabled(bool): Disables OOM-killer on the Docker container.
            default_allocation(int): Specify allocation(s) instead of using the
                    Pterodactyl deployment service.  Uses the allocation's
                    internal ID and not the port number.
            additional_allocations(iter): Additional allocations on top of
                    default_allocation.
        """
        if default_allocation is None and not location_ids:
            raise BadRequestError('Must specify either default_allocation or '
                                  'location_ids')

        # Fetch the Egg variables which are required to create the server.
        egg_info = self._api_request(
            endpoint='application/nests/%s/eggs/%s' % (
                nest_id, egg_id), params={'include': 'variables'})['attributes']
        egg_vars = egg_info['relationships']['variables']['data']

        # Build a dict of environment variables.  Prefer values passed in the
        # environment parameter, otherwise use the default value from the Egg
        # config.
        env_with_defaults = {}
        for var in egg_vars:
            var_name = var['attributes']['env_variable']
            if var_name in environment:
                env_with_defaults[var_name] = environment[var_name]
            else:
                env_with_defaults[var_name] = var['attributes'].get(
                    'default_value')

        if not docker_image:
            docker_image = egg_info.get('docker_image')
        if not startup_cmd:
            startup_cmd = egg_info.get('startup')

        data = {
            'name': name,
            'user': user_id,
            'nest': nest_id,
            'egg': egg_id,
            'docker_image': docker_image,
            'startup': startup_cmd,
            'oom_disabled': oom_disabled,
            'limits': {
                'memory': memory_limit,
                'swap': swap_limit,
                'disk': disk_limit,
                'io': io_limit,
                'cpu': cpu_limit,
            },
            'feature_limits': {
                'databases': database_limit,
                'allocations': allocation_limit,
                'backups': backup_limit
            },
            'environment': env_with_defaults,
            'start_on_completion': start_on_completion,
        }

        if default_allocation is not None:
            data['allocation'] = {'default': default_allocation,
                                  'additional': additional_allocations}
        elif location_ids:
            data['deploy'] = {'locations': location_ids,
                              'dedicated_ip': dedicated_ip,
                              'port_range': port_range}

        response = self._api_request(endpoint='application/servers',
                                     mode='POST', data=data, json=False)
        return response

    def update_server_build(self, server_id, allocation_id, memory_limit=None,
                            swap_limit=None, disk_limit=None, cpu_limit=None,
                            io_limit=None, database_limit=None,
                            allocation_limit=None, backup_limit=None, add_allocations=None,
                            remove_allocations=None, oom_disabled=None):
        """Updates the build configuration for an existing server.

        Modifies an existing server identified by allocation_id and updates
        any parameters that are passed.

        *** WARNING ***
        This endpoint has a lot of requirements and it doesn't always surface
        helpful errors.  Sometimes they're in the panel logs.  I plan to
        automate some of the painful parts so you can specify only the fields
        you want to update, however currently you must satisfy the API's
        requirements by passing in everything.

        Example of a working set of parameters:
            update_server_build(server_id=12, allocation_id=81, 
                    memory_limit=2048, swap_limit=2048, disk_limit=5120, 
                    cpu_limit=100, io_limit=500, database_limit=1, 
                    allocation_limit=2, backup_limit=4, 
                    add_allocations=None, remove_allocations=None, 
                    oom_disabled=True)

        Args:
            server_id(int): Internal server ID, e.g. 12
            allocation_id(int): Base allocation of the server to modify.
            memory_limit(int): Memory limit in MB for the Docker container.  To
                    allow unlimited memory limit set to 0.
            swap_limit(int): Swap limit in MB for the Docker container.  To not
                    assign any swap set to 0.  For unlimited swap set to -1.
            disk_limit(int): Disk limit in MB for the Docker container.  To
                    allow unlimited disk space set to 0.
            cpu_limit(int): CPU limit for the Docker container.  To allow
                    unlimited CPU usage set to 0.  To limit to one core set
                    to 100.  For four cores set to 400.
            io_limit(int): Block IO weight for the Docker container.
                    Must be between 10 and 1000.
            database_limit(int): Maximum number of databases that can be
                    assigned to this server.
            allocation_limit(int): Maximum number of allocations that can be
                    assigned to this server.
            backup_limit(int): Maximum number of backups that can be
                    assigned to this server.
            add_allocations(iter): List of allocation IDs to add to the server.
            remove_allocations(iter): List of allocation IDs to remove from
                    the server.
            oom_disabled(bool): Disables OOM-killer on the Docker container.
        """
        data = {
            'allocation': allocation_id,
            'limits': {},
            'feature_limits': {},
        }

        if memory_limit is not None:
            data['limits']['memory'] = memory_limit
        if swap_limit is not None:
            data['limits']['swap'] = swap_limit
        if disk_limit is not None:
            data['limits']['disk'] = disk_limit
        if cpu_limit is not None:
            data['limits']['cpu'] = cpu_limit
        if io_limit is not None:
            data['limits']['io'] = io_limit
        if database_limit is not None:
            data['feature_limits']['databases'] = database_limit
        if allocation_limit is not None:
            data['feature_limits']['allocations'] = allocation_limit
        if backup_limit is not None:
            data['feature_limits']['backups'] = backup_limit
        if add_allocations is not None:
            data['add_allocations'] = add_allocations
        if remove_allocations is not None:
            data['remove_allocations'] = remove_allocations
        if oom_disabled is not None:
            data['oom_disabled'] = oom_disabled

        response = self._api_request(
            endpoint='application/servers/%s/build' % server_id,
            mode='PATCH', data=data, json=False)
        return response

    def update_server_startup(self, server_id, egg_id=None,
                              environment={}, docker_image=None,
                              startup_cmd=None, skip_scripts=None):
        """Updates the startup config for the specified server.

        Modifies the startup config of an existing server replacing any
        specified values.  Unspecified values will not be changed.

        Args:
            egg_id(int): Egg ID to update on the server.
            environment(dict): Key value pairs of Service Variables to set.
                    Every variable from the egg must be set or the API will
                    return an error.  Any keys specified will be overwritten
                    in the existing environment list.  Unspecified keys will
                    not be modified. Extra keys will be dropped.
            docker_image(str): Name or URL of the Docker server to use.
                    e.g. quay.io/pterodactyl/core:java-glibc
            startup_cmd(str): Startup command, if specified replaces the
                    egg's default value.
            skip_scripts(bool): True to skip egg scripts.
        """
        server_info = self._api_request(
            endpoint='application/servers/%s' % server_id,
            params={'include':'variables'})['attributes']
        container = server_info['container']
        current_env = container['environment']
        current_egg = server_info['egg']
        merged_env = {}
        if egg_id is not None and egg_id != current_egg:
            nest_id = server_info['nest']
            egg_info = self._api_request(
                endpoint='application/nests/%s/eggs/%s' % (
                    nest_id, egg_id),
                params={'include': 'variables'})['attributes']
            egg_vars = egg_info['relationships']['variables']['data']

            # Build a dict of environment variables. Prefer values passed in
            # the environment parameter, then those set in the current env,
            # then finally use the default value from the Egg config.
            for var in egg_vars:
                var_name = var['attributes']['env_variable']
                if var_name in environment:
                    merged_env[var_name] = environment[var_name]
                elif var_name in current_env:
                    merged_env[var_name] = current_env[var_name]
                else:
                    merged_env[var_name] = var['attributes'].get('default_value')
            if not docker_image:
                docker_image = egg_info.get('docker_image')
            if not startup_cmd:
                startup_cmd = egg_info.get('startup')
        elif environment is not None:
            # If extra values are passed they are silently dropped by the API
            #   could detect attempts to set invalid variables and throw.
            # Also could reduce the contents of current_env down to the
            #   set of variables, there's extra info in here that seems
            #   to not matter if it's set or not.
            merged_env = current_env
            merged_env.update(environment)

        data = {
            'egg': egg_id if egg_id is not None else current_egg,
            'startup': startup_cmd if startup_cmd is not None else container['startup_command'],
            'image': docker_image if docker_image is not None else container['image'],
            # Cant find a way to get the current value for this setting
            #   and this seems better than assuming true or false
            'skip_scripts': skip_scripts if skip_scripts is not None else container['installed'] == 1,
            'environment': merged_env
        }

        response = self._api_request(
            endpoint='application/servers/%s/startup' % server_id,
            mode='PATCH', data=data, json=False)
        return response


import unittest
from unittest import mock

from pydactyl import PterodactylClient


class SettingsTests(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_rename_server(self, mock_api):
        expected = {
            'endpoint': 'client/servers/f1d2s3/settings/rename',
            'mode': 'POST',
            'data': {'name': 'newname'},
        }
        self.api.client.servers.settings.rename_server('f1d2s3', 'newname')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_reinstall_server(self, mock_api):
        expected = {
            'endpoint': 'client/servers/f1d2s3/settings/reinstall',
            'mode': 'POST',
        }
        self.api.client.servers.settings.reinstall_server('f1d2s3')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



from pydactyl.api import base
from pydactyl.constants import POWER_SIGNALS
from pydactyl.exceptions import BadRequestError
from pydactyl.responses import PaginatedResponse


class ServersBase(base.PterodactylAPI):
    """Pterodactyl Client Server Base API.

    Methods in this class appear in the base **client.servers** namespace
    when using PterodactylClient.
    """

    def list_servers(self):
        """List all servers the client has access to."""
        endpoint = 'client'
        response = self._api_request(endpoint=endpoint)
        return PaginatedResponse(self, endpoint, response)

    def list_permissions(self):
        """Retries all available permissions.

        This is used by the frontend.  I have no idea what this does.
        """
        endpoint = 'client/permissions'
        response = self._api_request(endpoint=endpoint)
        return response

    def get_server(self, server_id, detail=False):
        """Get information for the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            detail(bool): If True includes the object type and a nested data
                    structure.  This is not particularly useful.
        """
        response = self._api_request(
            endpoint='client/servers/{}'.format(server_id))
        return base.parse_response(response, detail)

    def get_server_utilization(self, server_id, detail=False):
        """Get resource utilization information for the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            detail(bool): If True includes the object type and a nested data
                    structure.  This is not particularly useful.
        """
        response = self._api_request(
            endpoint='client/servers/{}/resources'.format(server_id))
        return base.parse_response(response, detail)

    def send_console_command(self, server_id, cmd):
        """Sends a console command to the specified server.

        The server must be online, otherwise API will return a HTTP 412 error.
        If successful, there will be an empty response body.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            cmd(str): Console command to send to the server
        """
        data = {'command': cmd}
        endpoint = 'client/servers/{}/command'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data, json=False)
        return response

    def send_power_action(self, server_id, signal):
        """Sends a console command to the specified server.

        The server must be online, otherwise API will return a HTTP 412 error.
        If successful, there will be an empty response body.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            signal(str): Power signal to send to the server.
                Valid options:
                start - Sends the startup command to the server.
                stop - Sends the stop command to the server.
                restart - Stops the server then immediately starts it.
                kill - Instantly ends all processes and marks the server as
                       stopped.  The kill signal can corrupt server files
                       and should only be used as a last resort.
        """
        if signal not in POWER_SIGNALS:
            raise BadRequestError(
                'Invalid power signal sent({}), must be one of: {}'.format(
                    signal, POWER_SIGNALS))

        data = {'signal': signal}
        endpoint = 'client/servers/{}/power'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST',
                                     data=data, json=False)
        return response

    def get_websocket(self, server_id):
        """Generates credentials to connect to the server's websocket.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        Returns:
            response(dict): Response with token and websocket address
        """
        endpoint = 'client/servers/{}/websocket'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='GET')
        return response



from pydactyl.api import base


class Network(base.PterodactylAPI):
    """Pterodactyl Client Server Network API."""

    def list_allocations(self, server_id: str):
        """Retrieves network information for the specified server.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/network/allocations'.format(server_id)
        response = self._api_request(endpoint=endpoint)
        return response

    def assign_allocation(self, server_id: str):
        """Assigns an allocation to the server.

        Automatically assigns a new allocation if auto-assign is enabled on
        the instance.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
        """
        endpoint = 'client/servers/{}/network/allocations'.format(server_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response

    def set_allocation_note(self, server_id: str, allocation_id: int,
                            note: str):
        """Sets the note on an allocation.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            allocation_id(int): Allocation identifier (e.g. 2)
            note(str): Contents of the note
        """
        data = {'notes': note}
        endpoint = 'client/servers/{}/network/allocations/{}'.format(
            server_id, allocation_id)
        response = self._api_request(endpoint=endpoint, mode='POST', data=data)
        return response

    def set_primary_allocation(self, server_id: str, allocation_id: int):
        """Sets the specified allocation as the primary allocation.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            allocation_id(int): Allocation identifier (e.g. 2)
        """
        endpoint = 'client/servers/{}/network/allocations/{}/primary'.format(
            server_id, allocation_id)
        response = self._api_request(endpoint=endpoint, mode='POST')
        return response

    def unassign_allocation(self, server_id: str, allocation_id: int):
        """Deletes the specified non-primary allocation.

        Args:
            server_id(str): Server identifier (abbreviated UUID)
            allocation_id(int): Allocation identifier (e.g. 2)
        """
        endpoint = 'client/servers/{}/network/allocations/{}'.format(
            server_id, allocation_id)
        response = self._api_request(endpoint=endpoint, mode='DELETE')
        return response



from unittest import main, mock, TestCase

from pydactyl import PterodactylClient
from pydactyl.exceptions import BadRequestError


class ServersTests(TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_servers(self, mock_api):
        expected = {
            'endpoint': 'application/servers',
        }
        self.client.servers.list_servers()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_server_info_by_external_id(self, mock_api):
        expected = {
            'endpoint': 'application/servers/external/11',
        }
        self.client.servers.get_server_info(external_id=11)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_server_info_by_server_id(self, mock_api):
        expected = {
            'endpoint': 'application/servers/22',
        }
        self.client.servers.get_server_info(server_id=22)
        mock_api.assert_called_with(**expected)

    def test_get_server_info_raises_with_no_id(self):
        with self.assertRaises(BadRequestError):
            self.client.servers.get_server_info()

    def test_get_server_info_raises_with_both_ids(self):
        with self.assertRaises(BadRequestError):
            self.client.servers.get_server_info(server_id=1, external_id=2)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_suspend_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/33/suspend',
            'mode': 'POST',
        }
        self.client.servers.suspend_server(server_id=33)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_unsuspend_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/44/unsuspend',
            'mode': 'POST',
        }
        self.client.servers.unsuspend_server(server_id=44)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_reinstall_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/55/reinstall',
            'mode': 'POST',
        }
        self.client.servers.reinstall_server(server_id=55)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_rebuild_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/66/rebuild',
            'mode': 'POST',
        }
        self.client.servers.rebuild_server(server_id=66)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/77',
            'mode': 'DELETE',
        }
        self.client.servers.delete_server(server_id=77)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_force_delete_server(self, mock_api):
        expected = {
            'endpoint': 'application/servers/88/force',
            'mode': 'DELETE',
        }
        self.client.servers.delete_server(server_id=88, force=True)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_server_databases(self, mock_api):
        expected = {
            'endpoint': 'application/servers/99/databases',
        }
        self.client.servers.list_server_databases(server_id=99)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_server_database_info(self, mock_api):
        expected = {
            'endpoint': 'application/servers/111/databases/5',
        }
        self.client.servers.get_server_database_info(server_id=111,
                                                     database_id=5)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_server_database(self, mock_api):
        expected = {
            'endpoint': 'application/servers/222/databases',
            'mode': 'POST',
        }
        self.client.servers.create_server_database(server_id=222)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_server_database(self, mock_api):
        expected = {
            'endpoint': 'application/servers/333/databases/6',
            'mode': 'DELETE',
        }
        self.client.servers.delete_server_database(server_id=333, database_id=6)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_reset_server_database_password(self, mock_api):
        expected = {
            'endpoint': 'application/servers/333/databases/6/reset-password',
            'mode': 'POST',
        }
        self.client.servers.reset_server_database_password(server_id=333,
                                                           database_id=6)
        mock_api.assert_called_with(**expected)

    def test_create_server_without_allocation_or_location_raises(self):
        with self.assertRaisesRegex(BadRequestError, 'default_allocation'):
            self.client.servers.create_server('test server', 1, 1, 1, 0, 0, 0)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_server_with_location(self, mock_api):
        self.client.servers.create_server('test server', 1, 2, 3, 4, 5, 6,
                                          location_ids=[7])
        mock_api.assert_called()

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_server_with_allocation(self, mock_api):
        self.client.servers.create_server('test server', 1, 2, 3, 4, 5, 6,
                                          default_allocation=1234)
        mock_api.assert_called()


if __name__ == '__main__':
    main()



import unittest
from unittest import mock

from pydactyl import PterodactylClient


class DatabasesTest(unittest.TestCase):

    def setUp(self):
        self.api = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_databases(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases',
            'params': {},
        }
        self.api.client.servers.databases.list_databases('fds173')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_databases_with_passwords(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases',
            'params': {'include': 'password'},
        }
        self.api.client.servers.databases.list_databases('fds173',
                                                         include_passwords=True)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_database(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases',
            'mode': 'POST',
            'data': {'database': 'testdb', 'remote': '%'},
        }
        self.api.client.servers.databases.create_database('fds173', 'testdb')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_create_database_with_remote(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases',
            'mode': 'POST',
            'data': {'database': 'testdb', 'remote': '1.1.1.1'},
        }
        self.api.client.servers.databases.create_database('fds173', 'testdb',
                                                          '1.1.1.1')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_rotate_database_password(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases/longlong/rotate'
                        '-password',
            'mode': 'POST',
        }
        self.api.client.servers.databases.rotate_database_password('fds173',
                                                                   'longlong')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_database(self, mock_api):
        expected = {
            'endpoint': 'client/servers/fds173/databases/longlong',
            'mode': 'DELETE',
        }
        self.api.client.servers.databases.delete_database('fds173', 'longlong')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()
